#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "vector.h"

static _Bool
vector_realloc (struct vector *v)
{
	if (!v)
		return 0;

	v->capacity *= 2;
	v->elements = realloc (v->elements, sizeof (void *) * v->capacity);
	return (_Bool)v->elements;
}

struct vector
vector_create ()
{
	struct vector v = { 0 };

	v.capacity = VECTOR_INIT_CAPACITY;
	v.elements = calloc (v.capacity, sizeof (void *));
	if (!v.elements)
		fputs ("vector_create: cannot allocate memory.", stderr);

	return v;
}

_Bool
vector_push_back (void *elm, struct vector *v)
{
	if (!elm || !v)
		return 0;

	if ((v->length + 1) >= v->capacity)
		if (!vector_realloc (v))
			return 0;

	v->elements[v->length] = elm;
	v->length++;

	return 1;
}

void *
vector_at (size_t i, struct vector *v)
{
	if (!v)
		return NULL;

	if (i >= v->length)
		{
			fprintf (stderr,
							 "vector_at: cannot access index '%lu' in an array of '%lu' "
							 "elements.\n",
							 i, v->length + 1);
		}

	return v->elements[i];
}

void
vector_free (struct vector *v)
{
	if (!v)
		return;

	free (v->elements);
	memset (v, 0, sizeof (struct vector));
}
